$:.unshift File.expand_path('../../lib', __FILE__)

require 'pp'
require "active_support/all"

require "sms/version"
require "sms/zucp"
require "sms/taobao"

require 'redis' unless defined? Redis
require 'yaml' unless defined? YMAL

def redis
  if $redis && $redis.class.to_s =~ /Redis/
    $redis
  else
    Redis.new(host: 'localhost', port: 6379)
  end
end

def load_config
  if defined? ::Rails
    @sms_config ||= HashWithIndifferentAccess.new(YAML.load_file("#{::Rails.root}/config/sms.yml")[::Rails.env] || {})
  else
    {}
  end
end

class SmsSender

  SOURCES = %w(taobao zucp)

  attr_accessor :source, :template_code

  def initialize(options = {})
    options[:source] = redis.get('sms:source') || SOURCES.first if options[:source].blank?
    return unless options[:source].in?(SOURCES)

    if load_config.present? && load_config[options[:source]].present?
      options = load_config[options[:source]].merge(options)
    end
    options = HashWithIndifferentAccess.new(options)

    if options[:source].eql? 'zucp'

      if options[:template_code].present?
        templates = backend(I18n.locale => {'sms_zucp_templates' => load_config[:zucp][:templates]})
        I18n.backend = I18n::Backend::Chain.new(templates)

        options.merge!(content: I18n.t("sms_zucp_templates.#{options[:template_code].to_s.gsub('.', '_')}", options[:content_params]))
      end

    elsif options[:source].eql? 'taobao'

      if options[:template_code].present?
        options.merge!(sms_template_code: load_config[:taobao][:templates][options[:template_code].to_s.gsub('.', '_')])
        options.merge!(sms_param: options[:content_params])
      end

    end

    @source = "Sms/#{options[:source]}".camelize.constantize.new(options)
  end

  def singleSend(number)
    return unless @source
    return unless number.present?
    @source.singleSend(number)
  end

  def batchSend(numbers)
    return unless @source
    _numbers = numbers.is_a?(String) ? numbers.split(',') : (numbers.is_a?(Array) ? numbers : [])
    _numbers = _numbers.compact.uniq
    @source.batchSend(_numbers)
  end

  protected

  def backend(translations)
    backend = I18n::Backend::Simple.new
    translations.each { |locale, data| backend.store_translations(locale, data) }
    backend
  end

end

if defined? ::Rails
  module Sms
    module Rails
      class Railtie < ::Rails::Railtie
        rake_tasks do
          load "tasks/sms.rake"
        end
      end
    end
  end
end